import { Root } from 'hast';
import { Transformer } from 'unified';

type HighlightCallbackArgs = {
    /**
     * The language pulled from the fenced code block
     *
     * @example
     * ````md
     * ```js
     * console.log('hello world')
     * ```
     * ````
     *
     * Would result in the following language string `"js"`
     */
    lang: string | null | undefined;
    /**
     * The code within the fenced code block to be highlighted.
     *
     * @example
     * ````md
     * ```js
     * console.log('hello world')
     * ```
     * ````
     *
     * Would result in the following code string `"console.log('hello world')"`
     */
    value: string | undefined;
    /**
     * The metastring associated with the fenced block, which is the string
     * following the language declaration.
     *
     * @example
     * ````md
     * ```js title="example.md" showLineNumbers
     * console.log('hello world')
     * ```
     * ````
     *
     * Would result in the following metastring `"title=\"example.md\" showLineNumbers"`
     *
     */
    meta: string | null | undefined;
};
type MaybePromise<T> = T | Promise<T>;
type HighlightOptions = {
    highlight?: (args: HighlightCallbackArgs) => MaybePromise<string | undefined>;
};

type HastTransformer = Transformer<Root, Root>;
declare function rehypeCustomHighlight(options?: HighlightOptions): HastTransformer;

export { type HighlightCallbackArgs, type HighlightOptions, rehypeCustomHighlight };
