// ../common/src/constants.ts
var ENTITIES = {
  LEFT_CURLY: "&#123;",
  RIGHT_CURLY: "&#125;",
  LEFT_ANGLE: "&#60;",
  RIGHT_ANGLE: "&#62;",
  BACK_TICK: "&#96;",
  BACK_SLASH: "&#92;"
};

// ../common/src/index.ts
function escapeSvelte(str) {
  return str.replace(/[{}`]/g, (c) => {
    if (c === "{")
      return ENTITIES.LEFT_CURLY;
    if (c === "}")
      return ENTITIES.RIGHT_CURLY;
    if (c === "`")
      return ENTITIES.BACK_TICK;
  }).replace(/\\([trn])/g, "&#92;$1");
}

// src/index.ts
import { visit } from "unist-util-visit";

// src/utils.ts
function isElement(node) {
  return node ? node.type === "element" : false;
}
function getHighlightDataFromCode(element) {
  const lang = extractLangFromElement(element);
  const meta = extractMetaFromElement(element);
  const value = extractCodeContentsFromElement(element);
  return { lang, meta, value };
}
function extractLangFromElement(element) {
  if (element.properties && Array.isArray(element.properties.className) && typeof element.properties.className[0] === "string" && element.properties.className[0].startsWith("language-")) {
    return element.properties.className[0].replace("language-", "");
  }
  return void 0;
}
function extractMetaFromElement(element) {
  if (element.data && "meta" in element.data && typeof element.data.meta === "string") {
    return element.data.meta;
  }
  return void 0;
}
function extractCodeContentsFromElement(element) {
  if (!(element.children && element.children.length > 0 && element.children[0]))
    return;
  const textElement = element.children[0];
  if (textElement.type !== "text")
    return;
  return textElement.value.replace(/\n$/, "");
}

// src/index.ts
function rehypeCustomHighlight(options) {
  return async (tree) => {
    if (!options?.highlight)
      return;
    const nodesToHighlight = [];
    visit(tree, "element", (node) => {
      if (node.tagName !== "pre" || !Array.isArray(node.children))
        return;
      const codeEl = node.children[0];
      if (!isElement(codeEl) || codeEl.tagName !== "code")
        return;
      nodesToHighlight.push(codeEl);
    });
    const highlightNodes = async (element) => {
      const highlightData = getHighlightDataFromCode(element);
      const highlightedCode = await options?.highlight?.(highlightData);
      if (!highlightedCode)
        return;
      element.type = "raw";
      element.value = escapeSvelte(highlightedCode);
    };
    await Promise.all(nodesToHighlight.map((node) => highlightNodes(node)));
  };
}
export {
  rehypeCustomHighlight
};
//# sourceMappingURL=index.js.map